use telers::{
    enums::ParseMode,
    event::{telegram::HandlerResult, EventReturn},
    filters::CommandObject,
    types::Message,
    utils::text::{html_formatter::Formatter as HTMLFormatter, Builder as TextBuilder},
    Bot,
};

use crate::{
    handlers::actions::unmute::unmute,
    types::{
        enums::target_user::TargetUser,
        structs::{
            handler_entity::HandlerEntity,
            message_sender::{MessageSender, MessageSenderBuilder},
        },
    },
};

async fn unmute_help(sender: MessageSenderBuilder, bot: &Bot) {
    let help_response_builder: TextBuilder<HTMLFormatter> =
        TextBuilder::new(HTMLFormatter::default())
            .monowidth("/unmute [ID | REPLY]")
            .text(
                "
                \n\nВведите либо ID юзера, либо ответьте на \
                сообщение участника чата, с которого вы хотите \
                снять мьют. 
                ",
            );

    let help_response: &str = help_response_builder.get_text();

    sender
        .text(help_response)
        .parse_mode(ParseMode::HTML)
        .build()
        .send(bot)
        .await
        .unwrap();
}

pub async fn unmute_command(bot: Bot, message: Message, command: CommandObject) -> HandlerResult {
    let args: Vec<&str> = command.args.iter().map(|boxed| &**boxed).collect();

    let (message_id, chat_id): (i64, i64) = (message.id(), message.chat().id());
    let sender: MessageSenderBuilder = MessageSender::builder(chat_id).reply_to(message_id);

    let mut handler_entity: HandlerEntity = HandlerEntity {
        bot_instance: bot,
        message_reciever: message,
        message_sender_builder: sender,
    };

    match args.first().cloned() {
        None => {
            if handler_entity.message_reciever.reply_to_message().is_none() {
                unmute_help(
                    handler_entity.message_sender_builder,
                    &handler_entity.bot_instance,
                )
                .await;
            } else {
                let on_reply = TargetUser::Reply(handler_entity.message_reciever.clone());
                unmute(handler_entity, chat_id, on_reply).await?;
            }
        }
        Some(id) => {
            handler_entity
                .message_sender_builder
                .set_text("Укажите id пользователя, с которого вы хотите снять мьют");

            if let Ok(parsed_id) = id.parse::<i64>() {
                let on_id: TargetUser = TargetUser::Id(parsed_id);
                unmute(handler_entity, chat_id, on_id).await?;
            } else {
                handler_entity
                    .message_sender_builder
                    .build()
                    .send(&handler_entity.bot_instance)
                    .await
                    .unwrap();
            }
        }
    }

    Ok(EventReturn::Finish)
}
